/*
 * ---------------------------------------------------------------------------
 * This file defines constants, functions, ... used to navigate through the 
 * menue of the gasmixer.
 * author : Mstl Georg
 * date   : 16.11.2006
 * ---------------------------------------------------------------------------
 */
 
#include "menue_hdl.h"
#include "lcd_ctrl.h"
#include "keypad_ctrl.h"
#include "gasmixer_defs.h"
#include "mix_calc.h"

#include <stdint.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>


/* 
 * -------------------------------------------------------------------------
 * internal constants and variables declaration
 * -------------------------------------------------------------------------
 */

static char pad_str[MAX_LINE_LENGTH_LCD+1];
static char tmp_str[MAX_LINE_LENGTH_LCD+1];

static const char next_screen_str[] PROGMEM = 
	"*********  " RIGHT_ARROW_STR_LCD " next[#]";
static const char del_ret_str[]     PROGMEM = 
	"del[*]        ret[#]";
static const char back_screen_str[] PROGMEM = 
	"*********  " RIGHT_ARROW_STR_LCD " back[#]";
	

static const char unit_lit_str[] PROGMEM = "l";
static const char unit_bar_str[] PROGMEM = "bar";
static const char unit_per_str[] PROGMEM = "%";

static const char qual_old_str[] PROGMEM = "old";
static const char qual_new_str[] PROGMEM = "new";
static const char qual_max_str[] PROGMEM = "max";
static const char qual_min_str[] PROGMEM = "min";
static const char qual_inp_str[] PROGMEM = "inp";

static const char gas_O2_str[]  PROGMEM = "O2";
static const char gas_He_str[]  PROGMEM = "He";
static const char gas_Top_str[] PROGMEM = "Top";

static const char cfg_store_tank_str[]     PROGMEM = "Storing tank %s: %c";
static const char cfg_err_store_tank_str[] PROGMEM = "Err store tank %s: %c";
static const char cfg_topoff_gas_str[]     PROGMEM = "Topoff gas part: %s";
static const char cfg_err_topoff_gas_str[] PROGMEM = "Err topoff: %s";

static const char calc_src_tank_str[]      PROGMEM = "Source tank: %s";
static const char calc_err_src_tank_str[]  PROGMEM = "Err src tank: %s";
static const char calc_dest_tank_str[]     PROGMEM = "Destination tank: %s";
static const char calc_err_dest_tank_str[] PROGMEM = "Err dest tank: %s";

/*static const char cfg_val_fmt_str[MAX_LENGTH_FMT_STR+1]    PROGMEM = 
  "%3.3s [%.3s] = %3u";*/
static const char fill_instr_fmt_str[] =
  "Fill %3.3u %-3.3s get %3.3u";
	
static const char cfg_val_fmt_str[] = 
  "%3.3s [%.3s] = %3u";
	
	
/* 
 * -------------------------------------------------------------------------
 * internal function declaration
 * -------------------------------------------------------------------------
 */

uint16_t setValMENUE(const uint8_t start_pos, const uint8_t old_val);

void entCfgValScreenMENUE(const char* title, const char* unit, 
											    const uint8_t old_val);

void entCalcValScreenMENUE(const char* title, const char* unit, 
                           const uint8_t max_val, const uint8_t def_val); 
													 
void entValMaxErrScreenMENUE(const char* title, const char* unit, 
                             const uint16_t inp_val, const uint8_t max_val);
														 
void entValMinErrScreenMENUE(const char* title, const char* unit, 
                             const uint16_t inp_val, const uint8_t min_val);
													
bool checkValMENUE(const char* title, const char* unit, const uint16_t val, 
                   const uint8_t max_val, const uint8_t min_val);

void emptySrcTankScreenMENUE(const uint8_t old_val, const uint8_t new_val);

void renewStoreTankScreenMENUE(const char* gas);

void fillInstrScreenMENUE(fill_instr_t instr);

void padStrMENUE(const char* s, const pad_dir_t pad_dir, const char pad_char);


/* 
 * -------------------------------------------------------------------------
 * internal function definitions
 * -------------------------------------------------------------------------
 */
/*
uint16_t setValMENUE(const uint8_t start_pos, const uint8_t old_val) {

  const uint8_t dig_offset = 0x30;
	
  //uint8_t key     = readKeypadKEY();
	uint8_t key     = readKeypadKEY(&BUT_PORT, &BUT_PIN);
	uint8_t nr_key  = 0;
	uint8_t key_val = 0; //values 0 ... 9
	uint16_t val    = old_val;
	
	uint8_t key_arr[3] = {0};
	
	while((key != BUT_FENCE) && (key != BUT_MULT))
		key = readKeypadKEY(&BUT_PORT, &BUT_PIN);	
	
	if(key == BUT_FENCE) 
	  nr_key = 3;
		
	else {
		padStrMENUE("", right_e, PAD_CHAR);
		sendTextPosLCD(pad_str, THIRD_LINE_START_LCD + start_pos);
		val = 0;
	}
	
	waitReleasedKEY(key);
	
	//enter new value -> max. 3 digits allowed
	while(nr_key < 3) {
	
		key = readKeypadKEY(&BUT_PORT, &BUT_PIN);
		
		if(nr_key <= 0) 
			while(key == BUT_FENCE || key == BUT_MULT || key == NO_KEY)
				key = readKeypadKEY(&BUT_PORT, &BUT_PIN);
	  
		else 
			while(key == NO_KEY)
				key = readKeypadKEY(&BUT_PORT, &BUT_PIN);
		
		switch (key) {
		
			case BUT_FENCE: 
				//enter was pressed -> exit
				nr_key = 3; 
				waitReleasedKEY(key); 
				break;
		
		  case BUT_MULT:
				//delete was pressed -> enter new value
				nr_key = 0; 
				val    = 0;
				padStrMENUE("", right_e, PAD_CHAR);
				sendTextPosLCD(pad_str, THIRD_LINE_START_LCD + start_pos);
				waitReleasedKEY(key);
				break;
				
			default:
				//read next digit
				key_val = detKeyValKEY(key);
				//FIXXME
				val += key_val * 10 * nr_key;
				sendSignPosLCD(key_val + dig_offset, 
				               THIRD_LINE_START_LCD + start_pos + nr_key);
				waitReleasedKEY(key);
				nr_key++;	
		
		}//switch
			
	}//while
	
	return val;

}//setValMENUE


void entCfgValScreenMENUE(const char* title, const char* unit, 
                          const uint8_t old_val) {

	char u[MAX_LENGTH_UNIT_STR+1]; //unit string in RAM
	char q[MAX_LENGTH_QUAL_STR+1]; //qualifier string - " -
	
	//get strings from flash
	strncpy_P(u, unit, MAX_LENGTH_UNIT_STR+1);
	
	//ruler   : --------------------(20 signs) -------------------- (20 signs)
	//1. line : <title>
	//2. line : <qual> [<unit>] = old_val  ->  old [bar] = 123   
	//3. line : <qual> [<unit>] = old_val  ->  new [bar] = 123
	//4. line : del[*]        ret[#]      	
	
	padStrMENUE(title, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	//fmt == "%3.3s [%.3s] = %3u"
	strncpy_P(q, qual_old_str, MAX_LENGTH_QUAL_STR+1);
	sprintf(tmp_str, cfg_val_fmt_str, q, u, old_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(q, qual_new_str, MAX_LENGTH_QUAL_STR+1);
	sprintf(tmp_str, cfg_val_fmt_str, q, u, old_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, del_ret_str, MAX_LINE_LENGTH_LCD+1);	
	sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
		
}//entCfgValScreenMENUE


void entCalcValScreenMENUE(const char* title, const char* unit, 
                           const uint8_t max_val, const uint8_t def_val) {

	char u[MAX_LENGTH_UNIT_STR+1]; //unit string in RAM
	char q[MAX_LENGTH_QUAL_STR+1]; //qualifier string - " -
	
	//get strings from flash
	strncpy_P(u, unit, MAX_LENGTH_UNIT_STR+1);
	
	//ruler   : --------------------(20 signs) -------------------- (20 signs)
	//1. line : <title>
	//2. line : <qual> [<unit>] = max_val  ->  max [bar] = 123   
	//3. line : <qual> [<unit>] = def_val  ->  inp [bar] = 123
	//4. line : del[*]        ret[#]      	
	
	padStrMENUE(title, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	//fmt == "%3.3s [%.3s] = %3u"
	strncpy_P(q, qual_max_str, MAX_LENGTH_QUAL_STR+1);
	sprintf(tmp_str, cfg_val_fmt_str, q, u, max_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(q, qual_inp_str, MAX_LENGTH_QUAL_STR+1);
	sprintf(tmp_str, cfg_val_fmt_str, q, u, def_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, del_ret_str, MAX_LINE_LENGTH_LCD+1);	
	sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
	
}//entCalcValScreenMENUE


void entValMaxErrScreenMENUE(const char* title, const char* unit, 
                             const uint16_t inp_val, const uint8_t max_val) {
	
	char u[MAX_LENGTH_UNIT_STR+1]; //unit string in RAM
	char q[MAX_LENGTH_QUAL_STR+1]; //qualifier string - " -
	
	//get strings from flash
	strncpy_P(u, unit, MAX_LENGTH_UNIT_STR+1);
	
	//ruler   : -------------------- (20 signs)	-------------------- (20 signs)
	//1. line : <title>
	//2. line : <qual> [<unit>] = max_val		->  max [bar] = 123     
	//3. line : <qual> [<unit>] = inp_val		->  inp [bar] = 321
	//4. line :           -> back[#]
	
	padStrMENUE(title, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	//fmt == "%3.3s [%.3s] = %3u"
	strncpy_P(q, qual_max_str, MAX_LENGTH_QUAL_STR+1);
	sprintf(tmp_str, cfg_val_fmt_str, q, u, max_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(q, qual_inp_str, MAX_LENGTH_QUAL_STR+1);
	sprintf(tmp_str, cfg_val_fmt_str, q, u, inp_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, del_ret_str, MAX_LINE_LENGTH_LCD+1);	
	sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
		
}//entValMaxErrScreenMENUE


void entValMinErrScreenMENUE(const char* title, const char* unit, 
                             const uint16_t inp_val, const uint8_t min_val) {
	
	char u[MAX_LENGTH_UNIT_STR+1]; //unit string in RAM
	char q[MAX_LENGTH_QUAL_STR+1]; //qualifier string - " -
	
	//get strings from flash
	strncpy_P(u, unit, MAX_LENGTH_UNIT_STR+1);
	
	//ruler   : -------------------- (20 signs)	-------------------- (20 signs)
	//1. line : <title>
	//2. line : <qual> [<unit>] = min_val		->  min [bar] = 123     
	//3. line : <qual> [<unit>] = inp_val		->  inp [bar] = 321
	//4. line :           -> back[#]
	
	padStrMENUE(title, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	//fmt == "%3.3s [%.3s] = %3u"
	strncpy_P(q, qual_min_str, MAX_LENGTH_QUAL_STR+1);
	sprintf(tmp_str, cfg_val_fmt_str, q, u, min_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(q, qual_inp_str, MAX_LENGTH_QUAL_STR+1);
	sprintf(tmp_str, cfg_val_fmt_str, q, u, inp_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, del_ret_str, MAX_LINE_LENGTH_LCD+1);	
	sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
				
}//entValMinErrScreenMENUE


bool checkValMENUE(const char* title, const char* unit, const uint16_t val, 
                   const uint8_t max_val, const uint8_t min_val) {
	
	uint8_t key; 
	
	if(val > max_val) 
		entValMaxErrScreenMENUE(title, unit, val, max_val);
	
	else if(val < min_val)
		entValMinErrScreenMENUE(title, unit, val, min_val);
		
	else
		return true;
		
//	key = readKeypadKEY();
	key = readKeypadKEY(&BUT_PORT, &BUT_PIN);
	while(key != BUT_FENCE)
	  key = readKeypadKEY(&BUT_PORT, &BUT_PIN);
	
	clearLCD();
	waitReleasedKEY(BUT_FENCE);
	
	return false;

}//checkValMENUE
 
 
void emptySrcTankScreenMENUE(const uint8_t old_val, const uint8_t new_val) {

	char u[MAX_LENGTH_UNIT_STR+1]; //unit string copied into RAM
	char q[MAX_LENGTH_QUAL_STR+1]; //qualifier string - " -
	
	//get strings from flash
	strncpy_P(u, unit_bar_str, MAX_LENGTH_UNIT_STR+1);
	
	//ruler   : -------------------- (20 signs)
	//1. line : Empty src tank!
	//2. line : old [bar] = old_val    
	//3. line : new [bar] = new_val
	//4. line : ********* -> next[#]
	strncpy_P(tmp_str, PSTR("Empty src tank!"), MAX_LINE_LENGTH_LCD+1);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);

	//fmt == "%3.3s [%.3s] = %3u"
	strncpy_P(q, qual_old_str, MAX_LENGTH_QUAL_STR+1);	
	sprintf(tmp_str, cfg_val_fmt_str, q, u, old_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(q, qual_new_str, MAX_LENGTH_QUAL_STR+1);	
	sprintf(tmp_str, cfg_val_fmt_str, q, u, new_val);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, next_screen_str, MAX_LINE_LENGTH_LCD+1);
  sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
		
}//emptySrcTankScreenMENUE
 

void renewStoreTankScreenMENUE(const char* gas) {

	char g[MAX_LENGTH_GAS_STR+1];
	
	//ruler   : -------------------- (20 signs)
	//1. line : Renew O2 store tank!
	//2. line : Make new cfg and new
	//3. line : calculation!
	//4. line : ********* -> next[#]
	strncpy_P(tmp_str, PSTR("Renew %s store tank!"), MAX_LINE_LENGTH_LCD+1);
	strncpy_P(g, gas, MAX_LENGTH_GAS_STR+1);
	sprintf(tmp_str, tmp_str, g);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("Make new cfg and new"), MAX_LINE_LENGTH_LCD+1);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("calculation!"), MAX_LINE_LENGTH_LCD+1);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, next_screen_str, MAX_LINE_LENGTH_LCD+1);
	sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
	
}//renewStoreTankScreenMENUE

 
void fillInstrScreenMENUE(fill_instr_t instr) {

	char g[MAX_LENGTH_GAS_STR+1];  //used gas string
	
	uint8_t inc_p = instr.p + instr.amount.am_O2;
	
	//ruler   : -------------------- (20 signs)
	//1. line : Fill xxx O2  get yyy
	//2. line : Fill xxx He  get yyy
	//3. line : Fill xxx Top get yyy
	//4. line : del[*]        ret[#]
	
	//fmt == "Fill %3.3u %-3.3s get %3.3u";
	strncpy_P(g, gas_O2_str, MAX_LENGTH_GAS_STR+1);
	sprintf(tmp_str, fill_instr_fmt_str, instr.amount.am_O2, g, inc_p);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	inc_p += instr.amount.am_He;
	strncpy_P(g, gas_He_str, MAX_LENGTH_GAS_STR+1);
	sprintf(tmp_str, fill_instr_fmt_str, instr.amount.am_He, g, inc_p);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
		
	inc_p += instr.amount.am_Top;
	strncpy_P(g, gas_Top_str, MAX_LENGTH_GAS_STR+1);
	sprintf(tmp_str, fill_instr_fmt_str, instr.amount.am_Top, g, inc_p);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, del_ret_str, MAX_LINE_LENGTH_LCD+1);
  sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
		
}//fillInstrScreenMENUE
*/

void padStrMENUE(const char* s, const pad_dir_t pad_dir, 
                 const char pad_char) {

	uint8_t i, dif;
	
	if (strlen(s) >= MAX_LINE_LENGTH_LCD) {
		
		//no padding needed -> substring
		for(i=0; i<MAX_LINE_LENGTH_LCD; i++)
			pad_str[i] = s[i];
	  
		pad_str[i] = '\0';
		
	} else {
		
		//strlen(s) < MAX_LINE_LENGTH_LCD -> padding needed
		dif = MAX_LINE_LENGTH_LCD - strlen(s);
		if (pad_dir == left_e) {
		
			for(i=0; i<dif; i++)
				pad_str[i] = pad_char;
			
			for(; i<MAX_LINE_LENGTH_LCD+1; i++)
			  pad_str[i] = s[i-dif];
				
			//'\0' from s is copied into pad_str
				
		} else {
		
			//pad on the right	
			for(i=0; s[i] != '\0'; i++)
				pad_str[i] = s[i];
			
			for(; i<MAX_LINE_LENGTH_LCD; i++)
				pad_str[i] = pad_char;
				
			pad_str[i] = '\0';
			
		}//else
				
	}//else

}//padStrMENUE
 
 
/* 
 * -------------------------------------------------------------------------
 * general function definitions
 * -------------------------------------------------------------------------
 */
	 
void welcomeScreenMENUE(void) {
 
	//create welcome screen
	//ruler   : -------------------- (20 signs)
	//1. line : ********************
	//2. line : Gasmixer HW v1.0    
	//3. line : by Moestl Georg     
	//4. line : ********* -> next[#]
	
	padStrMENUE("", right_e, '*');
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("Gasmixer HW v1.0"), MAX_LINE_LENGTH_LCD+1);	
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("by Moestl Georg"), MAX_LINE_LENGTH_LCD+1);	
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, next_screen_str, MAX_LINE_LENGTH_LCD+1);	
	sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
						 
}//welcomeScreenMENUE
 
 
void cfgScreenMENUE(void) {

	//create configuration screen
	//ruler   : -------------------- (20 signs)
	//1. line : Configuration of  
	//2. line : V, p, TopOff, ...
	//3. line : Do you want some?
	//4. line : no[*]         yes[#]
	
	strncpy_P(tmp_str, PSTR("Configuration of"), MAX_LINE_LENGTH_LCD+1);	
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("V, p, TopOff, ..."), MAX_LINE_LENGTH_LCD+1);	
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("Do you want some?"), MAX_LINE_LENGTH_LCD+1);	
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("no[*]         yes[#]"), MAX_LINE_LENGTH_LCD+1);	
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, FOURTH_LINE_START_LCD);
	
}//cfgScreenMENUE 

/*
void makeNewCfgMENUE(void) {

  extern cfg_par_t cfg_par;
	
	cfg_par_t par;
	uint16_t  val;
	
	char gas[MAX_LENGTH_GAS_STR+1];

	//read old configuration from eeprom
	eeprom_read_block(&par, &cfg_par, sizeof(cfg_par_t));

	//set pressure of storing tank O2 [bar]
	//ruler   : -------------------- (20 signs)
	//1. line : Storing tank O2: p
	//2. line : old [bar] = xxx     
	//3. line : new [bar] = yyy
	//4. line : del[*]        ret[#] 
	
	//get gas string from flash
	strncpy_P(gas, gas_O2_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Storing tank %s: %c")
		strncpy_P(tmp_str, cfg_store_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas, 'p');	
	  entCfgValScreenMENUE(tmp_str, unit_bar_str, par.p_store_tank[IDX_O2]);	
	
		//create error screen (tmp_str == "Err store tank %s: %c")
	  strncpy_P(tmp_str, cfg_err_store_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas, 'p');
	  
		//read user input
		val = setValMENUE(12, par.p_store_tank[IDX_O2]);
	
	} while(!checkValMENUE(tmp_str, unit_bar_str, val, 
	                       MAX_VAL_P_STORE, MIN_VAL_P_STORE));
	
	par.p_store_tank[IDX_O2] = val;
	clearLCD();
	
	//set pressure of storing tank He [bar]
	//ruler   : -------------------- (20 signs)
	//1. line : Storing tank He: p
	//2. line : old [bar] = xxx     
	//3. line : new [bar] = yyy
	//4. line : del[*]        ret[#] 
	
	//get gas string from flash
	strncpy_P(gas, gas_He_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Storing tank %s: %c")
		strncpy_P(tmp_str, cfg_store_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas, 'p');	
	  entCfgValScreenMENUE(tmp_str, unit_bar_str, par.p_store_tank[IDX_HE]);	
	
		//create error screen (tmp_str == "Err store tank %s: %c")
	  strncpy_P(tmp_str, cfg_err_store_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas, 'p');
	  
		//read user input
		val = setValMENUE(12, par.p_store_tank[IDX_HE]);
	
	} while(!checkValMENUE(tmp_str, unit_bar_str, val, 
	                       MAX_VAL_P_STORE, MIN_VAL_P_STORE));
		
	par.p_store_tank[IDX_HE] = val;
	clearLCD();
	
	//set volume of storing tank O2[l]
	//ruler   : -------------------- (20 signs)
	//1. line : Storing tank O2: V
	//2. line : old [l] = xxx     
	//3. line : new [l] = yyy
	//4. line : del[*]        ret[#] 
	
	//get gas string from flash
	strncpy_P(gas, gas_O2_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Storing tank %s: %c")
		strncpy_P(tmp_str, cfg_store_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas, 'V');	
	  entCfgValScreenMENUE(tmp_str, unit_lit_str, par.V_store_tank[IDX_O2]);	
	
		//create error screen (tmp_str == "Err store tank %s: %c")
	  strncpy_P(tmp_str, cfg_err_store_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas, 'V');
	  
		//read user input
		val = setValMENUE(10, par.V_store_tank[IDX_O2]);
	
	} while(!checkValMENUE(tmp_str, unit_lit_str, val, 
	                       MAX_VAL_V_STORE, MIN_VAL_V_STORE));
		
	par.V_store_tank[IDX_O2] = val;
	clearLCD();
	
	//set volume of storing tank He[l]
	//ruler   : -------------------- (20 signs)
	//1. line : Storing tank He: V
	//2. line : old [l] = xxx     
	//3. line : new [l] = yyy
	//4. line : del[*]        ret[#] 
	
	//get gas string from flash
	strncpy_P(gas, gas_He_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Storing tank %s: %c")
		strncpy_P(tmp_str, cfg_store_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas, 'V');	
	  entCfgValScreenMENUE(tmp_str, unit_lit_str, par.V_store_tank[IDX_HE]);	
	
		//create error screen (tmp_str == "Err store tank %s: %c")
	  strncpy_P(tmp_str, cfg_err_store_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas, 'V');
	  
		//read user input
		val = setValMENUE(10, par.V_store_tank[IDX_HE]);
	
	} while(!checkValMENUE(tmp_str, unit_lit_str, val, 
	                       MAX_VAL_V_STORE, MIN_VAL_V_STORE));
												 
	par.V_store_tank[IDX_HE] = val;
	clearLCD();
	
	//set volume of diving tank [l]
	//ruler   : -------------------- (20 signs)
	//1. line : Diving tank: max. V
	//2. line : old [l] = xxx     
	//3. line : new [l] = yyy
	//4. line : del[*]        ret[#] 
	do {
		
		//create and display enter screen (tmp_str == "Diving tank: max. V")
		strncpy_P(tmp_str, PSTR("Diving tank: max. V"), MAX_LINE_LENGTH_LCD+1);
		entCfgValScreenMENUE(tmp_str, unit_lit_str, par.V_dive_tank);
		
		//create error screen (tmp_str == "Err dive tank: V")
		strncpy_P(tmp_str, PSTR("Err dive tank: V"), MAX_LINE_LENGTH_LCD+1);
		
		//read user input
		val = setValMENUE(10, par.V_dive_tank);
		
  } while(!checkValMENUE(tmp_str, unit_lit_str, val, 
	                       MAX_VAL_V_DIVE, MIN_VAL_V_DIVE));
	
	par.V_dive_tank = val;
	clearLCD();	
	
	//set topoff mix O2 [%]
	//ruler   : -------------------- (20 signs)
	//1. line : Topoff gas part: O2
	//2. line : old [%] = xxx     
	//3. line : new [%] = yyy
	//4. line : del[*]        ret[#] 	
	
	//get gas string from flash
	strncpy_P(gas, gas_O2_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Topoff gas part: %s")
		strncpy_P(tmp_str, cfg_topoff_gas_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas);	
	  entCfgValScreenMENUE(tmp_str, unit_per_str, par.topoff.per_O2);	
	
		//create error screen (tmp_str == "Err topoff: %s")
	  strncpy_P(tmp_str, cfg_err_topoff_gas_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas);
	  
		//read user input
		val = setValMENUE(10, par.topoff.per_O2);
	
	} while(!checkValMENUE(tmp_str, unit_per_str, val, 
	                       MAX_VAL_GAS, MIN_VAL_GAS));
		
	par.topoff.per_O2 = val;
	clearLCD();

	//set topoff mix He [%]
	//ruler   : -------------------- (20 signs)
	//1. line : Topoff gas part: He
	//2. line : old [%] = xxx     
	//3. line : new [%] = yyy
	//4. line : del[*]        ret[#]
	
	//get gas string from flash
	strncpy_P(gas, gas_He_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Topoff gas part: %s")
		strncpy_P(tmp_str, cfg_topoff_gas_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas);	
	  entCfgValScreenMENUE(tmp_str, unit_per_str, par.topoff.per_He);	
	
		//create error screen (tmp_str == "Err topoff: %s")
	  strncpy_P(tmp_str, cfg_err_topoff_gas_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas);
	  
		//read user input
		val = setValMENUE(10, par.topoff.per_He);
	
	} while(!checkValMENUE(tmp_str, unit_per_str, val, 
	                       MAX_VAL_GAS - par.topoff.per_O2, MIN_VAL_GAS));
													 
	par.topoff.per_He = val;
	clearLCD();
	
	//set topoff mix N2 [%]
	par.topoff.per_N2 = MAX_VAL_GAS - par.topoff.per_O2 - par.topoff.per_He;
	
	//set shutoff pressure compressor [bar]
	//ruler   : -------------------- (20 signs)
	//1. line : Compressor: max. p
	//2. line : old [bar] = xxx     
	//3. line : new [bar] = yyy
	//4. line : del[*]        ret[#] 
	
	do {
		
		//create and display enter screen (tmp_str == "Compressor: max. p")
		strncpy_P(tmp_str, PSTR("Compressor: max. p"), MAX_LINE_LENGTH_LCD+1);
		entCfgValScreenMENUE(tmp_str, unit_bar_str, par.p_off_comp);
		
		//create error screen (tmp_str == "Err dive tank: V")
		strncpy_P(tmp_str, PSTR("Err compressor: p"), MAX_LINE_LENGTH_LCD+1);
		
		//read user input
		val = setValMENUE(12, par.p_off_comp);
		
  } while(!checkValMENUE(tmp_str, unit_bar_str, val, 
	                       MAX_VAL_P_COMP, MIN_VAL_P_COMP));
	
	par.p_off_comp = val;
	clearLCD();
	
	//write new configuration into eeprom
	eeprom_write_block(&par, &cfg_par, sizeof(cfg_par_t));
	
}//makeNewCfgMENUE


void calcScreenMENUE(void) {

	//create calculation screen
	//ruler   : -------------------- (20 signs)
	//1. line : ********************
	//2. line : Now the calculation 
	//3. line : can start! 
	//4. line : ********* -> next[#]
	
	padStrMENUE("", right_e, '*');
	sendTextPosLCD(pad_str, FIRST_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("Now the calculation"), MAX_LINE_LENGTH_LCD+1);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, SECOND_LINE_START_LCD);
	
	strncpy_P(tmp_str, PSTR("can start!"), MAX_LINE_LENGTH_LCD+1);
	padStrMENUE(tmp_str, right_e, PAD_CHAR);
	sendTextPosLCD(pad_str, THIRD_LINE_START_LCD);
	
	strncpy_P(tmp_str, next_screen_str, MAX_LINE_LENGTH_LCD+1);
	sendTextPosLCD(tmp_str, FOURTH_LINE_START_LCD);
	
}//calcScreenMENUE 


void makeNewCalcMENUE(void) {
	
	uint16_t val;
	tank_t   src;        //source tank
	tank_t   dest;       //destination tank
	uint8_t  key;
	
	fill_instr_t instr;
	
	extern cfg_par_t cfg_par;
	cfg_par_t par;
	
	char gas[MAX_LENGTH_GAS_STR+1];
	
	//get storing tank pressure
	eeprom_read_block(&par, &cfg_par, sizeof(cfg_par_t));
	
	//--------------------------------------------------------------------------
	// source tank
	//--------------------------------------------------------------------------
	
	//set pressure of source tank [bar]
	//ruler   : -------------------- (20 signs)
	//1. line : Source tank: p  
	//2. line : max [bar] = xxx
	//3. line : inp [bar] = yyy
	//4. line : del[*]        ret[#] 
		
	do {	
	
    //create and display enter screen (tmp_str == "Source tank: %s")
		strncpy_P(tmp_str, calc_src_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, "p");	
	  entCalcValScreenMENUE(tmp_str, unit_bar_str, MAX_VAL_P_DIVE, 
		                      P_DIVING_TANK);	
	
		//create error screen (tmp_str == "Err src tank: %s")
	  strncpy_P(tmp_str, calc_err_src_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, "p");
	  
		//read user input
		val = setValMENUE(12, P_DIVING_TANK);
		
	} while(!checkValMENUE(tmp_str, unit_bar_str, val, 
	                       MAX_VAL_P_DIVE-1, MIN_VAL_P_DIVE));
												 
	src.p = val;
	clearLCD();
	
	
	//set source tank gas mix O2 [%]
	//ruler   : -------------------- (20 signs)
	//1. line : Source tank: O2
	//2. line : max [%] = xxx     
	//3. line : inp [%] = yyy
	//4. line : del[*]        ret[#] 
	
	//get gas string from flash
	strncpy_P(gas, gas_O2_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Source tank: %s")
		strncpy_P(tmp_str, calc_src_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas);	
	  entCalcValScreenMENUE(tmp_str, unit_per_str, MAX_VAL_GAS, TOPOFF_O2);	
	
		//create error screen (tmp_str == "Err src tank: %s")
	  strncpy_P(tmp_str, calc_err_src_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas);
	  
		//read user input
		val = setValMENUE(10, TOPOFF_O2);
		
	} while(!checkValMENUE(tmp_str, unit_per_str, val, 
	                       MAX_VAL_GAS, MIN_VAL_GAS));
	
	src.gas.per_O2 = val;
	clearLCD();

	//set source tank gas mix He [%]
	//ruler   : -------------------- (20 signs)
	//1. line : Source tank: He
	//2. line : max [%] = xxx     
	//3. line : inp [%] = yyy
	//4. line : del[*]        ret[#] 
	
	//get gas string from flash
	strncpy_P(gas, gas_He_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Source tank: %s")
		strncpy_P(tmp_str, calc_src_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas);	
	  entCalcValScreenMENUE(tmp_str, unit_per_str, 
		                      MAX_VAL_GAS - src.gas.per_O2, TOPOFF_HE);	
	
		//create error screen (tmp_str == "Err src tank: %s")
	  strncpy_P(tmp_str, calc_err_src_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas);
	  
		//read user input
		val = setValMENUE(10, TOPOFF_HE);
		
	} while(!checkValMENUE(tmp_str, unit_per_str, val, 
	                       MAX_VAL_GAS - src.gas.per_O2, MIN_VAL_GAS));
	
	src.gas.per_He = val;
	clearLCD();

  //set source tank gas mix N2 [%]
	src.gas.per_N2 = MAX_VAL_GAS - src.gas.per_O2 - src.gas.per_He;
	
	
	//--------------------------------------------------------------------------
	// destination tank
	//--------------------------------------------------------------------------
	
	//set pressure of destination tank [bar]
	//ruler   : -------------------- (20 signs)
	//1. line : Destination tank: p  
	//2. line : max [bar] = xxx
	//3. line : inp [bar] = yyy
	//4. line : del[*]        ret[#] 
	
	do {	
	
    //create and display enter screen (tmp_str == "Destination tank: %s")
		strncpy_P(tmp_str, calc_dest_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, "p");	
	  entCalcValScreenMENUE(tmp_str, unit_bar_str, MAX_VAL_P_DIVE, 
		                      P_DIVING_TANK);	
	
		//create error screen (tmp_str == "Err dest tank: %s")
	  strncpy_P(tmp_str, calc_err_dest_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, "p");
	  
		//read user input
		val = setValMENUE(12, P_DIVING_TANK);
		
	} while(!checkValMENUE(tmp_str, unit_bar_str, val, 
	                       MAX_VAL_P_DIVE, MIN_VAL_P_DIVE+2));
												 
	dest.p = val;
	clearLCD();	
	
	//set destination tank gas mix O2 [%]
	//ruler   : -------------------- (20 signs)
	//1. line : Destination tank: O2
	//2. line : max [%] = xxx     
	//3. line : inp [%] = yyy
	//4. line : del[*]        ret[#] 
	
	//get gas string from flash
	strncpy_P(gas, gas_O2_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Destination tank: %s")
		strncpy_P(tmp_str, calc_dest_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas);	
	  entCalcValScreenMENUE(tmp_str, unit_per_str, MAX_VAL_GAS, TOPOFF_O2);	
	
		//create error screen (tmp_str == "Err dest tank: %s")
	  strncpy_P(tmp_str, calc_err_dest_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas);
	  
		//read user input
		val = setValMENUE(10, TOPOFF_O2);
		
	} while(!checkValMENUE(tmp_str, unit_per_str, val, 
	                       MAX_VAL_GAS, MIN_VAL_GAS));
	
	dest.gas.per_O2 = val;
	clearLCD();

	//set destination tank gas mix He [%]
	//ruler   : -------------------- (20 signs)
	//1. line : Destination tank: He
	//2. line : max [%] = xxx     
	//3. line : inp [%] = yyy
	//4. line : del[*]        ret[#] 
	
	//get gas string from flash
	strncpy_P(gas, gas_He_str, MAX_LENGTH_GAS_STR+1);
	
	do {	
	
    //create and display enter screen (tmp_str == "Destination tank: %s")
		strncpy_P(tmp_str, calc_dest_tank_str, MAX_LINE_LENGTH_LCD+1);
		sprintf(tmp_str, tmp_str, gas);	
	  entCalcValScreenMENUE(tmp_str, unit_per_str, 
		                      MAX_VAL_GAS - dest.gas.per_O2, TOPOFF_HE);	
	
		//create error screen (tmp_str == "Err dest tank: %s")
	  strncpy_P(tmp_str, calc_err_dest_tank_str, MAX_LINE_LENGTH_LCD+1);
	  sprintf(tmp_str, tmp_str, gas);
	  
		//read user input
		val = setValMENUE(10, TOPOFF_HE);
		
	} while(!checkValMENUE(tmp_str, unit_per_str, val, 
	                       MAX_VAL_GAS - dest.gas.per_O2, MIN_VAL_GAS));
	
	dest.gas.per_He = val;
	clearLCD();

  //set destination tank gas mix N2 [%]
	dest.gas.per_N2 = MAX_VAL_GAS - dest.gas.per_O2 - dest.gas.per_He;
	
	//calculate gasmix
	instr = calcGasmixCALC(src, dest);
	
	if(src.p != instr.p)
		//src tank has to be emptied before mix can be made
		emptySrcTankScreenMENUE(src.p, instr.p);
		
	//check storing tanks
	par.p_store_tank[IDX_O2] = 
		(uint8_t) 
		( ((par.p_store_tank[IDX_O2]*par.V_store_tank[IDX_O2]) -
			 (instr.amount.am_O2 * dest.gas.per_O2 * par.V_dive_tank / 100.0)) 
			 / par.V_store_tank[IDX_O2] );

	par.p_store_tank[IDX_HE] = 
		(uint8_t) 
		( ((par.p_store_tank[IDX_HE]*par.V_store_tank[IDX_HE]) -
			 (instr.amount.am_He * dest.gas.per_He * par.V_dive_tank / 100.0)) 
			 / par.V_store_tank[IDX_HE] );
			 
	if((instr.p + instr.amount.am_O2) > par.p_store_tank[IDX_O2]) { 
		
		renewStoreTankScreenMENUE(gas_O2_str);
		//wait until "enter" button (== '#') is pressed 
		while((key = readKeypadKEY(&BUT_PORT, &BUT_PIN)) != BUT_FENCE);
		clearLCD();
		waitReleasedKEY(key);
		
	} else if((instr.p + instr.amount.am_O2 + instr.amount.am_He) > 
	           par.p_store_tank[IDX_HE]) { 
		
		renewStoreTankScreenMENUE(gas_He_str);
		//wait until "enter" button (== '#') is pressed 
		while((key = readKeypadKEY(&BUT_PORT, &BUT_PIN)) != BUT_FENCE);
		clearLCD();
		waitReleasedKEY(key);
	
	} else {
	
	  //fill tank
	  fillInstrScreenMENUE(instr);
		//wait pressed key
	  key = readKeypadKEY(&BUT_PORT, &BUT_PIN);
		while((key != BUT_FENCE) && (key != BUT_MULT))
			key = readKeypadKEY(&BUT_PORT, &BUT_PIN);
		if(key == BUT_FENCE)
			//update storing tanks
			eeprom_write_block(&par, &cfg_par, sizeof(cfg_par_t));
		clearLCD();
		waitReleasedKEY(key);	
	
	}//else
		
}//makeNewCalcMENUE

*/

